@@ -57,7 +57,7 @@
 #' 
 register_font <- function(name, plain, bold = plain, italic = plain, bolditalic = plain, features = font_feature()) {
   if (name %in% system_fonts()$family) {
-    stop("A system font with that family name already exist", call. = FALSE)
+    stop("A system font with that family name already exists", call. = FALSE)
   }
   if (is.character(plain)) plain <- list(plain, 0)
   if (is.character(bold)) bold <- list(bold, 0)
@@ -107,7 +107,7 @@ clear_registry <- function() {
 #' 
 #' @examples
 #' # Get the default "sans" family
-#' sans <- match_font("sans")$path
+#' sans <- match_fonts("sans")$path
 #' sans <- system_fonts()$family[system_fonts()$path == sans][1]
 #' 
 #' # Register a variant of it:
@@ -145,10 +145,10 @@ register_variant <- function(name, family, weight = NULL, width = NULL, features
     bold <- which.min(abs(as.integer(sys_fonts$weight) - 7))
     weight <- sys_fonts$weight[unique(c(normal, bold))]
   }
-  plain <- sys_fonts[sys_fonts$weight == weight[1] & !sys_fonts$italic, , drop = FALSE]
-  bold <- if (length(weight) == 2) sys_fonts[sys_fonts$weight == weight[2] & !sys_fonts$italic, , drop = FALSE] else plain
-  italic <- sys_fonts[sys_fonts$weight == weight[1] & sys_fonts$italic, , drop = FALSE]
-  bolditalic <- if (length(weight) == 2) sys_fonts[sys_fonts$weight == weight[2] & sys_fonts$italic, , drop = FALSE] else italic
+  plain <- sys_fonts[which(sys_fonts$weight == weight[1] & !sys_fonts$italic), , drop = FALSE]
+  bold <- if (length(weight) == 2) sys_fonts[which(sys_fonts$weight == weight[2] & !sys_fonts$italic), , drop = FALSE] else plain
+  italic <- sys_fonts[which(sys_fonts$weight == weight[1] & sys_fonts$italic), , drop = FALSE]
+  bolditalic <- if (length(weight) == 2) sys_fonts[which(sys_fonts$weight == weight[2] & sys_fonts$italic), , drop = FALSE] else italic
   if (nrow(plain) == 0) plain <- italic
   if (nrow(bold) == 0) bold <- plain
   if (nrow(italic) == 0) italic <- plain
